<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Sirv Limited <support@sirv.com>
 *  @copyright Copyright (c) 2017-2023 Sirv Limited. All rights reserved
 *  @license   https://www.magictoolbox.com/license/
 */
class SirvAPIClient
{
    private $clientId = '';
    private $clientSecret = '';
    private $clientId_default = '';
    private $clientSecret_default = '';
    private $token = '';
    private $tokenExpireTime = 0;
    private $connected = false;
    private $lastResponse;
    private $userAgent;
    private $curlInfo;

    public function __construct(
        $clientId,
        $clientSecret,
        $clientId_default,
        $clientSecret_default,
        $token,
        $tokenExpireTime,
        $userAgent
    ) {
        $this->clientId = $clientId;
        $this->clientSecret = $clientSecret;
        $this->clientId_default = $clientId_default;
        $this->clientSecret_default = $clientSecret_default;
        $this->token = $token;
        $this->tokenExpireTime = $tokenExpireTime;
        $this->userAgent = $userAgent;
        $this->preOperationCheck();
    }

    public function fetchImage($imgs)
    {

        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $res = $this->sendRequest(
            'v2/files/fetch',
            $imgs,
            'POST'
        );

        if ($res) {
            $this->connected = true;
            return $res;
        } else {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
    }

    public function configFetching($url, $status, $minify)
    {

        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $data = array();

        if ($status) {
            $data = array(
                'minify' => array(
                    "enabled" => $minify
                ),
                'fetching' => array(
                    "enabled" => true,
                    "type" => "http",
                    "http" => array(
                        "url" => $url,
                    ),
                )
            );
        } else {
            $data = array(
                // 'minify' => array(
                //     "enabled" => false
                // ),
                // 'fetching' => array(
                //     "enabled" => false
                // )
            );
        }

        $res = $this->sendRequest('v2/account', $data, 'POST');

        if ($res) {
            $this->connected = true;
            return true;
        } else {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
    }

    public function allowDangerousContentTypes()
    {

        $res = $this->sendRequest('v2/account/dangerousContentTypes', array('allow'=>true), 'POST');

        if ($res) {
            $this->connected = true;
            return true;
        } else {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
    }


    public function configCDN($status, $alias)
    {

        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $data = array(
            'aliases' => array(
                $alias => array(
                    "cdn" => $status
                )
            )
        );

        $res = $this->sendRequest('v2/account', $data, 'POST');

        if ($res) {
            $this->connected = true;
            return true;
        } else {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
    }

    public function preOperationCheck()
    {
        if (empty($this->token) || $this->isTokenExpired()) {
            $this->getNewToken();
        }

        if ($this->connected) {
            return true;
        }

        if (empty($this->token) || $this->isTokenExpired()) {
            if (!$this->getNewToken()) {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    public function isConnected()
    {
        return $this->connected;
    }

    public function getNewToken()
    {
        if (empty($this->clientId) || empty($this->clientSecret)) {
            $this->nullClientLogin();
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
        $res = $this->sendRequest('v2/token', array(
            "clientId" => $this->clientId,
            "clientSecret" => $this->clientSecret,
        ));

        if ($res && $res->http_code == 200 && !empty($res->result->token) && !empty($res->result->expiresIn)) {
            $this->connected = true;
            $this->token = $res->result->token;
            $this->tokenExpireTime = time() + $res->result->expiresIn;
            $this->updateParentClassSettings();
            return $this->token;
        } else {
            $this->connected = false;
            if (!empty($res->http_code) && $res->http_code == 401) {
                $this->nullClientLogin();
            }
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }
    }

    private static function usersSortFunc($a, $b)
    {
        if ($a->alias == $b->alias) {
            return 0;
        }
        return ($a->alias < $b->alias) ? -1 : 1;
    }

    public function getUsersList($email, $password, $otpToken = '')
    {
        $res = $this->sendRequest('v2/token', array(
            "clientId" => $this->clientId_default,
            "clientSecret" => $this->clientSecret_default,
        ));

        if ($res && $res->http_code == 200 && !empty($res->result->token) && !empty($res->result->expiresIn)) {
            $data = [
                "email" => $email,
                "password" => $password,
            ];
            if ($otpToken!='') {
                $data["otpToken"] = $otpToken;
            }
            $res = $this->sendRequest('v2/user/accounts', $data, 'POST', $res->result->token);

            if ($res && $res->http_code == 417) {
                return 'OTPREQUIRES';
            }
            if ($res && $res->http_code == 200 && !empty($res->result) && is_array($res->result)) {
                uasort($res->result, array('SirvAPIClient', 'usersSortFunc'));
                $res->result = array_values($res->result);
                return $res->result;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    public function getFolderOptions($filename)
    {
        $res = $this->sendRequest(
            'v2/files/options?filename=/'.rawurlencode($filename).'&withInherited=true',
            array(),
            'GET'
        );
        if ($res && $res->http_code == 200) {
            return $res->result;
        } else {
            return false;
        }
    }
    
    public function getFileStat($filename)
    {

        $res = $this->sendRequest(
            'v2/files/stat?filename=/'.rawurlencode($filename),
            array(),
            'GET'
        );

        if ($res && $res->http_code == 200) {
            return $res->result;
        } else {
            return false;
        }
    }

    public function getProfiles()
    {

        $res = $this->sendRequest(
            'v2/files/readdir?dirname=/Profiles',
            array(),
            'GET'
        );

        if ($res && $res->http_code == 200) {
            return $res->result;
        } else {
            return false;
        }
    }
    
    public function setFolderOptions($filename, $options)
    {
        $res = $this->sendRequest(
            'v2/files/options?filename=/'.rawurlencode($filename),
            $options,
            'POST'
        );
        return ($res && $res->http_code == 200);
    }

    public function deleteFile($filename)
    {
        $res = $this->sendRequest(
            'v2/files/delete?filename=/'.rawurlencode($filename),
            array(),
            'POST'
        );

        return ($res && $res->http_code == 200);
    }

    public function rename($from, $to)
    {
        $res = $this->sendRequest(
            'v2/files/rename?from='.rawurlencode($from).'&to='.rawurlencode($to),
            array(),
            'POST'
        );
        return ($res && $res->http_code == 200);
    }

    public function uploadFile($fs_path, $sirv_path, $data = '')
    {

        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $res = $this->sendRequest(
            'v2/files/upload?filename='.$sirv_path,
            (!empty($data))?$data:Tools::file_get_contents($fs_path),
            'POST',
            '',
            true
        );
        return ($res && $res->http_code == 200);
    }

    public function registerAccount($email, $password, $firstName, $lastName, $alias)
    {
        $res = $this->sendRequest('v2/token', array(
            "clientId" => $this->clientId_default,
            "clientSecret" => $this->clientSecret_default,
        ));

        if ($res && $res->http_code == 200 && !empty($res->result->token) && !empty($res->result->expiresIn)) {
            $res = $this->sendRequest('v2/account', array(
                "email" => $email,
                "password" => $password,
                "firstName" => $firstName,
                "lastName" => $lastName,
                "alias" => $alias,
            ), 'PUT', $res->result->token);

            if ($res && $res->http_code == 200) {
                return true;
            } else {
                return false;
            }
        }
    }

    public function setupClientCredentials($token)
    {
        $res = $this->sendRequest('v2/rest/credentials', array(), 'GET', $token);
        if ($res && $res->http_code == 200 && !empty($res->result->clientId) && !empty($res->result->clientSecret)) {
            $this->clientId = $res->result->clientId;
            $this->clientSecret = $res->result->clientSecret;
            $this->getNewToken();
            $this->updateParentClassSettings();
            return true;
        } else {
            return false;
        }
    }

    public function setupS3Credentials($email = '')
    {
        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $res = $this->sendRequest('v2/account/users', array(), 'GET');

        if ($res && $res->http_code == 200 && !empty($res->result) && is_array($res->result) && count($res->result)) {
            $res_user = false;

            foreach ($res->result as $user) {
                $tmp_res = $this->sendRequest('v2/user?userId=' . $user->userId, array(), 'GET');
                if ($tmp_res && $tmp_res->http_code == 200 && $tmp_res->result->email == $email) {
                    $res_user = $tmp_res;
                    break;
                }
            }

            if ($res_user && $res_user->http_code == 200 &&
                !empty($res_user->result->s3Secret) && !empty($res_user->result->email)) {
                $res_alias = $this->sendRequest('v2/account', array(), 'GET');

                if ($res_alias && $res_alias->http_code == 200 &&
                    !empty($res_alias->result) && !empty($res_alias->result->alias)) {
                    $this->updateParentClassSettings(array(
                        'SIRV_S3_BUCKET' => $res_alias->result->alias,
                        'SIRV_S3_ACCESS_KEY' => $res_user->result->email,
                        'SIRV_S3_SECRET_KEY' => $res_user->result->s3Secret,
                    ));
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
            return true;
        } else {
            $this->updateParentClassSettings(array(
                'SIRV_S3_BUCKET' => '',
                'SIRV_S3_ACCESS_KEY' => '',
                'SIRV_S3_SECRET_KEY' => '',
            ));
            return false;
        }
    }

    public function updateParentClassSettings($extra_options = array())
    {
        if (class_exists('Configuration') && method_exists('Configuration', 'updateValue')) {
            Configuration::updateValue('SIRV_CLIENT_ID', $this->clientId);
            Configuration::updateValue('SIRV_CLIENT_SECRET', $this->clientSecret);
            Configuration::updateValue('SIRV_TOKEN', $this->token);
            Configuration::updateValue('SIRV_TOKEN_EXPIRE_TIME', $this->tokenExpireTime);
            if (count($extra_options)) {
                foreach ($extra_options as $option => $value) {
                    Configuration::updateValue($option, $value);
                }
            }
        }
        return true;
    }

    public function nullClientLogin()
    {
        $this->clientId = '';
        $this->clientSecret = '';
        $this->updateParentClassSettings(array(
            'SIRV_S3_BUCKET' => '',
            'SIRV_S3_ACCESS_KEY' => '',
            'SIRV_S3_SECRET_KEY' => '',
        ));
    }

    public function nullToken()
    {
        $this->token = '';
        $this->tokenExpireTime = 0;
    }

    public function isTokenExpired()
    {
        return $this->tokenExpireTime < time();
    }

    public function getAccountInfo()
    {
        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $result = $this->sendRequest('v2/account', array(), 'GET');

        if (!$result || empty($result->result) || $result->http_code != 200 || empty($result->result)) {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }

        return $result->result;
    }

    public function getSearchResults($q, $maxCount = 0, $from = 0, $doScroll = true, $returnResults = false)
    {

        $contents = []; $size = 100; $total = 0; $scrollId = '';

        if ($maxCount > 0 && $size > $maxCount) {
            $size = $maxCount;
        }

        if (!empty($doScroll) && $doScroll !== true) {
            $scrollId = $doScroll;
        }
        do {    
            if (!empty($scrollId)) {
                $res = $this->sendRequest(
                    'v2/files/search/scroll',
                    [
                        'scrollId' => $scrollId
                    ],
                    'POST'
                );                
            } else {
                $res = $this->sendRequest(
                    'v2/files/search',
                    [
                        'query' => $q, 
                        'from' => $from, 
                        'size' => $size, 
                        'scroll' => !empty($doScroll)
                    ],
                    'POST'
                );      
            }

            if ($returnResults) {
                $hasResults = count($res->result->hits)>0;
                return [ 'results' => $hasResults?array_values($res->result->hits):[], 'total' => $res->result->total, 'scrollId' => $hasResults?$res->result->scrollId:'' ];
            }

            if (empty($total) && $res && $res->http_code == 200 && isset($res->result) && isset($res->result->total)) {
                $total = $res->result->total;
                if ($maxCount == 0) {
                    $maxCount = $total;
                }
            }
            if ($res && $res->http_code == 200 && isset($res->result) && isset($res->result->scrollId)) {
                $scrollId = $res->result->scrollId;
            } else {
                $scrollId = '';
            }
            if (!($res && $res->http_code == 200)) {
                return [ 'results' => $contents, 'total' => $total, 'scrollId' => $scrollId ];
            }

            $contents = array_merge($contents, array_values($res->result->hits));
            $from += count($res->result->hits);
            $continuation = $from < $maxCount && $from < $total && $maxCount != -1 && $total > 0;

        } while (!empty($continuation));

        return [ 'results' => $contents, 'total' => $total, 'scrollId' => $scrollId ];
    }    

    public function getStorageInfo()
    {
        $preCheck = $this->preOperationCheck();
        if (!$preCheck) {
            return false;
        }

        $storageInfo = array();

        $result = $this->sendRequest('v2/account', array(), 'GET');

        $result_storage = $this->sendRequest('v2/account/storage', array(), 'GET');

        if (!$result || empty($result->result) || $result->http_code != 200
            || !$result_storage->result || empty($result->result) || $result_storage->http_code != 200) {
            $this->connected = false;
            $this->nullToken();
            $this->updateParentClassSettings();
            return false;
        }

        $result = $result->result;
        $result_storage = $result_storage->result;

        if (isset($result->alias)) {
            $storageInfo['account'] = $result->alias;

            $billing = $this->sendRequest('v2/billing/plan', array(), 'GET');

            $billing->result->dateActive = preg_replace(
                '/.*([0-9]{4}\-[0-9]{2}\-[0-9]{2}).*/ims',
                '$1',
                $billing->result->dateActive
            );

            $planEnd = strtotime('+30 days', strtotime($billing->result->dateActive));
            $now = time();

            $datediff = (int) round(($planEnd - $now) / (60 * 60 * 24));

            $until = ($planEnd > $now) ? ' (' . $datediff . ' day' . ($datediff > 1 ? 's' : '') . ' left)' : '';

            if ($planEnd < $now) {
                $until = '';
            }

            $storageInfo['plan'] = array(
                'name' => $billing->result->name,
                'trial_ends' => preg_match('/trial/ims', $billing->result->name) ?
                    'until ' . date("j F", strtotime('+30 days', strtotime($billing->result->dateActive))) . $until
                    : '',
                'storage' => $billing->result->storage,
                'storage_text' => Sirv::getFormatedFileSize($billing->result->storage),
                'storage__extra_text' => (!empty($result_storage->extra))?
                Sirv::getFormatedFileSize($result_storage->extra):
                '',
                'dataTransferLimit' => isset($billing->result->dataTransferLimit) ?
                    $billing->result->dataTransferLimit : '',
                'dataTransferLimit_text' => isset($billing->result->dataTransferLimit) ?
                    Sirv::getFormatedFileSize($billing->result->dataTransferLimit) : '&#8734',
                'dataTransferExtra_text' => !empty($result->extraDataTransfer) ?
                    Sirv::getFormatedFileSize($result->extraDataTransfer) : '',
            );

            $storage = $this->sendRequest('v2/account/storage', array(), 'GET');

            $storage->result->plan = $storage->result->plan + $storage->result->extra;

            $storageInfo['storage'] = array(
                'allowance' => $storage->result->plan,
                'allowance_text' => Sirv::getFormatedFileSize($storage->result->plan),
                'used' => $storage->result->used,
                'available' => $storage->result->plan - $storage->result->used,
                'available_text' => Sirv::getFormatedFileSize($storage->result->plan - $storage->result->used),
                'available_percent' => number_format(
                    ($storage->result->plan - $storage->result->used) / $storage->result->plan * 100,
                    2,
                    '.',
                    ''
                ),
                'used_text' => Sirv::getFormatedFileSize($storage->result->used),
                'used_percent' => number_format($storage->result->used / $storage->result->plan * 100, 2, '.', ''),
                'files' => $storage->result->files,
            );

            $storageInfo['traffic'] = array(
                'allowance' => isset($billing->result->dataTransferLimit) ? $billing->result->dataTransferLimit : '',
                'allowance_text' => isset($billing->result->dataTransferLimit) ?
                Sirv::getFormatedFileSize($billing->result->dataTransferLimit) : '&#8734',
            );

            $dates = array(
                'This month' => array(
                    date("Y-m-01"),
                    date("Y-m-t"),
                ),
                date("F Y", strtotime("first day of -1 month")) => array(
                    date("Y-m-01", strtotime("first day of -1 month")),
                    date("Y-m-t", strtotime("last day of -1 month")),
                ),
                date("F Y", strtotime("first day of -2 month")) => array(
                    date("Y-m-01", strtotime("first day of -2 month")),
                    date("Y-m-t", strtotime("last day of -2 month")),
                ),
                date("F Y", strtotime("first day of -3 month")) => array(
                    date("Y-m-01", strtotime("first day of -3 month")),
                    date("Y-m-t", strtotime("last day of -3 month")),
                ),
            );

            $dataTransferLimit = isset($billing->result->dataTransferLimit) ?
            $billing->result->dataTransferLimit : PHP_INT_MAX;

            foreach ($dates as $label => $date) {
                $traffic = $this->sendRequest('v2/stats/http?from=' . $date[0] . '&to=' . $date[1], array(), 'GET');

                if (!$traffic || $traffic->http_code != 200) {
                    $this->connected = false;
                    $this->nullToken();
                    $this->updateParentClassSettings();
                    return false;
                }

                unset($traffic->http_code);

                $traffic = (array)$traffic->result;

                $storageInfo['traffic']['traffic'][$label]['size'] = 0;

                if (count($traffic)) {
                    foreach ($traffic as $v) {
                        $storageInfo['traffic']['traffic'][$label]['size'] += (isset($v->total->size))
                        ? $v->total->size : 0;
                    }
                }
                $storageInfo['traffic']['traffic'][$label]['percent'] = number_format(
                    $storageInfo['traffic']['traffic'][$label]['size'] / $dataTransferLimit * 100,
                    2,
                    '.',
                    ''
                );
                if ($storageInfo['traffic']['traffic'][$label]['percent'] > 100) {
                    $storageInfo['traffic']['traffic'][$label]['percent'] = 100;
                }
                $storageInfo['traffic']['traffic'][$label]['percent_reverse'] = number_format(
                    100 - $storageInfo['traffic']['traffic'][$label]['size'] / $dataTransferLimit * 100,
                    2,
                    '.',
                    ''
                );
                $storageInfo['traffic']['traffic'][$label]['size_text'] =
                Sirv::getFormatedFileSize($storageInfo['traffic']['traffic'][$label]['size']);
            }
        }

        $result = $this->sendRequest('v2/account/limits', array(), 'GET');

        if ($result && !empty($result->result) && $result->http_code == 200) {
            $storageInfo['limits'] = $result->result;
            $storageInfo['limits'] = (array)$storageInfo['limits'];
            $date = new DateTime();
            $timeZone = $date->getTimezone();
            foreach ($storageInfo['limits'] as $type => $value) {

                if (!in_array($type, ['rest:global','rest:post:files:delete','fetch:file','rest:post:files:upload'])) {
                    unset($storageInfo['limits'][$type]);
                    continue;
                }

                if (in_array($type, ['images:realtime','zips2:create'])) {
                    unset($storageInfo['limits'][$type]);
                    continue;
                }
                $storageInfo['limits'][$type] = (array)$value;
                $value = (array)$value;
                $dt = new DateTime('@'.$value['reset']);
                $dt->setTimeZone(new DateTimeZone($timeZone->getName()));
                $storageInfo['limits'][$type]['reset_str'] = $dt->format("H:i:s");
                if (empty($value['limit'])) {
                    $storageInfo['limits'][$type]['used'] = '0%';
                } else {
                    $storageInfo['limits'][$type]['used'] = (round($value['count']/$value['limit']*10000)/100).'%';
                }

                $storageInfo['limits'][$type]['type'] = $type;


                $storageInfo['limits_by_name'][$type] = $value['limit'];
            }
            $storageInfo['limits'] = array_chunk($storageInfo['limits'], ceil((int)count($storageInfo['limits'])/2));
        }

        return $storageInfo;
    }

    public function getLastResponse()
    {
        return $this->lastResponse;
    }

    public function muteRequests($timestamp)
    {
        Configuration::updateValue('SIRV_MUTE', $timestamp);
    }

    public function isMuted()
    {
        return ((int)Configuration::get('SIRV_MUTE') > time());
    }

    private function sendRequest($url, $data, $method = 'POST', $token = '', $isFile = false)
    {;

        if ($this->isMuted() && $url != 'v2/token') {
            $this->curlInfo = array('http_code' => 429);
            return false;
        }

        $curl = curl_init();
        $headers = array();
        if (!empty($token)) {
            $headers[] = "authorization: Bearer " . ((!empty($token)) ? $token : $this->token);
        } else {
            $headers[] = "authorization: Bearer " . $this->token;
        }
        if (!$isFile) {
            $headers[] = "content-type: application/json";
        }

        $response_headers = [];

        curl_setopt($curl, CURLOPT_HEADERFUNCTION,
          function($curl, $header) use (&$response_headers)
          {
            $len = strlen($header);
            $header = explode(':', $header, 2);
            if (count($header) < 2) // ignore invalid headers
              return $len;

            $response_headers[strtolower(trim($header[0]))][] = trim($header[1]);
            
            return $len;
          }
        );        

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.sirv.com/" . $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            //CURLOPT_TIMEOUT => 0.00000001,

            CURLOPT_CONNECTTIMEOUT => 0.1,
            CURLOPT_TIMEOUT => 0.1,

            CURLOPT_USERAGENT => $this->userAgent,

            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_POSTFIELDS => (!$isFile)?json_encode($data):$data,
            CURLOPT_HTTPHEADER => $headers,
        ));

        $result = curl_exec($curl);

        $info = curl_getinfo($curl);



        if ($info['http_code'] == 429 &&
            preg_match('/requests until ([0-9]{4}\-[0-9]{2}\-[0-9]{2}.*?\([a-z]{1,}\))/ims', $result, $m)) {
            $time = strtotime($m[1]);
            $this->muteRequests($time);
        }

        if (
            isset($response_headers['x-global-ratelimit-limit']) && 
            !empty($response_headers['x-global-ratelimit-limit'][0]) && 
            isset($response_headers['x-global-ratelimit-remaining']) && 
            isset($response_headers['x-global-ratelimit-remaining'][0]) && 
            $response_headers['x-global-ratelimit-limit'][0] > 0 &&
            $response_headers['x-global-ratelimit-remaining'][0] / $response_headers['x-global-ratelimit-limit'][0] <= 0.1
        ) {
            $info['http_code'] = 429;
            $this->muteRequests($response_headers['x-global-ratelimit-reset'][0]);
        }

        $response = (object) $info;
        $response->result = json_decode($result);
        $response->error = curl_error($curl);

        $this->lastResponse = $response;

        curl_close($curl);

        return $response;
    }
}
